/* -*- c++ -*- ***************************************************************/
/* Echtzeitsysteme                                                           */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/*          M U L T I _ L E V E L _ Q U E U E _ S C H E D U L E R _          */
/*                  D E F E R R A B L E _ S E R V E R                        */
/*                                                                           */
/*---------------------------------------------------------------------------*/
/* A multi level queue scheduler that is also capable of dealing with        */
/* deferrable servers.                                                       */
/*****************************************************************************/

#ifndef __multi_level_queue_scheduler_deferrable_server_h__
#define __multi_level_queue_scheduler_deferrable_server_h__

#include "devices/scheduler_implementation_timer.h"
#include "infra/types.h"
#include "infra/utils.h"
#include "thread/deferrable_server.h"
#include "thread/idle_thread.h"
#include "thread/multi_level_queue_scheduler.h"
#include "thread/abstract_multi_level_queue_scheduler_deferrable_server.h"

/**
 * \brief An extension of the multi level queue scheduler that is also capable
 *        of dealing with deferrable servers.
 *
 * To support deferrable servers, two things are necessary:
 *
 * - replenish the execution budgets at certain points in time
 * - monitor the execution budget of the deferrable servers and suspend a
 *   deferrable server, when its execution budget is exhausted
 *
 * The first part is implemented by computing the gcd among all replenishment
 * periods of all deferrable servers. The timer is then programmed to issue an
 * interrupt after any gcd time units.
 * To implement also the second requirement with the same timer, the scheduler
 * keeps track of the time that is left until the next replenishment interrupt.
 * Everytime a deferrable server is started time measurement is stopped and the
 * measured time is subtracted from a variable that keeps track of the time that
 * is left until the next replenishment interrupt. Then the timer is programmed
 * to issue an interrupt when the execution budget of the deferrable server is 
 * exhausted, then time measurement is started again. When a deferrable server
 * exits, yields or gets killed, time measurement is stopped and the time till
 * the next replenishment interrupt and the servers execution budget are updated
 * accordingly.
 **/
template < ezstubs_uint8 MAX_PR >
class Multi_Level_Queue_Scheduler_Deferrable_Server 
  : public Abstract_Multi_Level_Queue_Scheduler_Deferrable_Server
{
 private:
  Deferrable_Server* ready_servers_impl[MAX_PR];


public:

  /**
   * \brief Constructor
   **/
  Multi_Level_Queue_Scheduler_Deferrable_Server():Abstract_Multi_Level_Queue_Scheduler_Deferrable_Server(){
	ready_servers = ready_servers_impl;
	MAX_PRIO = MAX_PR;
  }

};

#endif /* __multi_level_queue_schedulerdeferrable_server_h__ */
